CREATE DATABASE MatRam_XPA
GO

USE MatRam_XPA
GO

-- -----------------------------------------------------
--  Creaci usuari
-- -----------------------------------------------------
CREATE LOGIN MatRam14 
    WITH PASSWORD = '1234';
CREATE USER MatRam14 FOR LOGIN MatRam14
GO 

GRANT ALL TO MatRam14
GO

CREATE SCHEMA xpa_esquema AUTHORIZATION MatRam14
  

-- -----------------------------------------------------
-- Table xpa_esquema.Usuaris
-- -----------------------------------------------------
CREATE  TABLE  xpa_esquema.Usuaris (
  idUsuari VARCHAR(10) NOT NULL ,
  Nom VARCHAR(45) NOT NULL ,
  Cognom1 VARCHAR(45) NOT NULL ,
  Cognom2 VARCHAR(45) NULL ,
  Password VARCHAR(8) NULL,
  PRIMARY KEY (idUsuari) )



-- -----------------------------------------------------
-- Table xpa_esquema.Empleat
-- -----------------------------------------------------
CREATE  TABLE  xpa_esquema.Empleat (
  idUsuari VARCHAR(10) NOT NULL ,
  NumEmpleat INT IDENTITY NOT NULL ,
  PRIMARY KEY (idUsuari, NumEmpleat) ,
  CONSTRAINT FK_EmpUsuari
    FOREIGN KEY (idUsuari )
    REFERENCES xpa_esquema.Usuaris (idUsuari )
    )



-- -----------------------------------------------------
-- Table xpa_esquema.Client
-- -----------------------------------------------------
CREATE  TABLE  xpa_esquema.Client (
  idUsuari VARCHAR(10) NOT NULL ,
  NumHabitacio INT NOT NULL ,
  Data DATE NULL,
  PRIMARY KEY (idUsuari, NumHabitacio) ,
  CONSTRAINT FK_CliUsuari
    FOREIGN KEY (idUsuari )
    REFERENCES xpa_esquema.Usuaris (idUsuari )    
    )



-- -----------------------------------------------------
-- Table xpa_esquema.Servei
-- -----------------------------------------------------
CREATE  TABLE  xpa_esquema.Servei (
  idServei INT IDENTITY NOT NULL ,
  Descripcio VARCHAR(150) NULL ,
  PRIMARY KEY (idServei) )



-- -----------------------------------------------------
-- Table xpa_esquema.Activitat
-- -----------------------------------------------------
CREATE  TABLE  xpa_esquema.Activitat (
  idActivitat INT IDENTITY NOT NULL ,
  Descripcio VARCHAR(150) NULL ,
  NumPlaces INT NULL ,
  Data DATE NULL,
  Hora TIME NULL,
  PRIMARY KEY (idActivitat) )



-- -----------------------------------------------------
-- Table xpa_esquema.Recurs
-- -----------------------------------------------------
CREATE  TABLE  xpa_esquema.Recurs (
  idRecurs INT IDENTITY NOT NULL ,
  Descripcio VARCHAR(150) NULL ,
  NumPlaces INT NULL ,
  PRIMARY KEY (idRecurs) )




-- -----------------------------------------------------
-- Table xpa_esquema.ReservaRecurs
-- -----------------------------------------------------
CREATE  TABLE  xpa_esquema.ReservaRecurs (
  Recurs INT NOT NULL ,
  Client VARCHAR(10) NOT NULL ,
  Data DATE NOT NULL ,
  Hora TIME NOT NULL ,
  PRIMARY KEY (Recurs, Client, Data, Hora) ,
    CONSTRAINT FK_RecRecurs
    FOREIGN KEY (Recurs )
    REFERENCES xpa_esquema.Recurs (idRecurs ),
  CONSTRAINT FK_RecClient
    FOREIGN KEY (Client )
    REFERENCES xpa_esquema.Usuaris (idUsuari )  
    )






-- -----------------------------------------------------
-- Table xpa_esquema.ReservaActivitat
-- -----------------------------------------------------
CREATE  TABLE  xpa_esquema.ReservaActivitat (
  Activitat INT NOT NULL ,
  Client VARCHAR(10) NOT NULL ,
  Places INT NULL ,
  PRIMARY KEY (Activitat, Client) ,
  CONSTRAINT FK_ActActivitat
    FOREIGN KEY (Activitat )
    REFERENCES xpa_esquema.Activitat (idActivitat ),
  CONSTRAINT FK_ActClient
    FOREIGN KEY (Client )
    REFERENCES xpa_esquema.Usuaris (idUsuari )    
    )





-- -----------------------------------------------------
-- Table xpa_esquema.PeticioServei
-- -----------------------------------------------------
CREATE  TABLE  xpa_esquema.PeticioServei (
  Servei INT NOT NULL ,
  Client VARCHAR(10) NOT NULL ,
  Data DATE NOT NULL ,
  Hora TIME NOT NULL ,
  PRIMARY KEY (Servei, Client, Data, Hora) ,
  CONSTRAINT FK_ServServei
    FOREIGN KEY (Servei )
    REFERENCES xpa_esquema.Servei (idServei ),
  CONSTRAINT FK_ServClient
    FOREIGN KEY (Client )
    REFERENCES xpa_esquema.Usuaris (idUsuari )    
    )



CREATE VIEW xpa_esquema.ConsultesAgenda AS
SELECT 'Activitat' as Tipus, idActivitat as Codi, Data, Hora, Descripcio, NumPlaces, idUsuari, Nom, Cognom1, Cognom2 FROM xpa_esquema.ReservaActivitat rAct
LEFT JOIN xpa_esquema.Activitat Act ON Act.idActivitat = rAct.Activitat LEFT JOIN xpa_esquema.Usuaris Us ON rAct.Client = Us.idUsuari
UNION
SELECT 'Recurs' as Tipus, idRecurs as Codi, Data, Hora, Descripcio, NumPlaces, idUsuari, Nom, Cognom1, Cognom2 FROM xpa_esquema.ReservaRecurs rRec
LEFT JOIN xpa_esquema.Recurs Rec ON Rec.idRecurs = rRec.Recurs  LEFT JOIN xpa_esquema.Usuaris Us ON rRec.Client = Us.idUsuari
UNION
SELECT 'Peticio' as Tipus, idServei as Codi, Data, Hora, Descripcio, 0 as NumPlaces, idUsuari, Nom, Cognom1, Cognom2 FROM xpa_esquema.PeticioServei rPet
LEFT JOIN xpa_esquema.Servei Pet ON Pet.idServei = rPet.Servei  LEFT JOIN xpa_esquema.Usuaris Us ON rPet.Client = Us.idUsuari



create view vClients as 
select a.NumHabitacio, a.Data, b.* from Client as a, Usuaris as b where a.idUsuari=b.idUsuari;

GO


USE MatRam_XPA
GO
ALTER USER MatRam14 WITH DEFAULT_SCHEMA = xpa_esquema;
GO

USE MatRam_XPA
GO
INSERT INTO xpa_esquema.Usuaris (idUsuari, Nom,Cognom1, Cognom2, Password) VALUES ('Admin','Administrador','SPA',NULL,'Admin');
GO
INSERT INTO xpa_esquema.Empleat (idUsuari) VALUES ('Admin');
GO



